@@ -3,7 +3,7 @@ as.xts.stars <- function(x,...) {
 	if (!requireNamespace("xts", quietly = TRUE))
 		stop("xts required: install that first") # nocov
 	if (length(x) > 1)
-		warning("all but first attribute will be ignored")
+		message("only the first attribute will used; maybe use merge() first?")
 
 	ed = expand_dimensions.stars(x)
 	time = which(sapply(ed, inherits, c("Date", "POSIXt")))
@@ -13,19 +13,23 @@ as.xts.stars <- function(x,...) {
 		message("more than one time dimension present in object; taking the first")
 		time = time[1]
 	}
-	x = st_upfront(x[1], time)
+	x = adrop(st_upfront(x[1], time), drop_xy = TRUE)
 	if (length(dim(x)) > 2) {
 		dims = setNames(c(dim(x)[1], prod(dim(x)[-1])), c(names(ed)[time], "other"))
 		x = st_redimension(x, dims)
+	} else { # set colnames
+		d = st_dimensions(x)
+		if (length(dim(x)) == 2 && length(d[[2]]$values) == dim(x)[2] && is.character(d[[2]]$values))
+			colnames(x[[1]]) = d[[2]]$values
 	}
-	
 	xts::xts(x[[1]], ed[[time]])
 }
 
 #' @name st_as_stars
+#' @param name character; attribute name for array from an \code{xts} object
 #' @details for the \code{xts} methods, if \code{dimensions} are provided, time has to be the first dimension.
 #' @export
-st_as_stars.xts = function(.x, ..., dimensions) {
+st_as_stars.xts = function(.x, ..., dimensions, name = "attr") {
 	if (!requireNamespace("xts", quietly = TRUE))
 		stop("xts required: install that first") # nocov
 	if (!requireNamespace("zoo", quietly = TRUE))
@@ -35,9 +39,9 @@ st_as_stars.xts = function(.x, ..., dimensions) {
 	if (!missing(dimensions)) {
 		.x = as.matrix(.x)
 		dim(.x) = dim(dimensions)
-		st_stars(list(.x), dimensions)
+		st_stars(setNames(list(.x), name), dimensions)
 	} else {
-		.x = st_as_stars(list(as.matrix(.x)))
+		.x = st_as_stars(setNames(list(as.matrix(.x)), name))
 		.x = st_set_dimensions(.x, 1, time)
 		if (!is.null(cn))
 			.x = st_set_dimensions(.x, 2, cn)
